;************************ RFM12 Sende-Routinen *****************************

;***************************** RFM12 Send CMD ******************************
; Sub: Sendet 16Bit Kommando aus Z
; Parameter:	Z Kommando
; Return:		Y Status ???
; Scratch-Reg:	Z, TEMP_0, TEMP_1
;***************************************************************************

// nFFS: 1-10k Pullup an Vcc !!!

RFM12_send_cmd:
		cbi		RFM12_PORT,RFM12_CS

		ldi		TEMP_1,16			; Anzahl Bits

RFM12_send_cmd_loop:
		cbi		RFM12_PORT,RFM12_SDI		
		sbrc	ZH,7
		sbi		RFM12_PORT,RFM12_SDI		

		nop
		nop
		sbi		RFM12_PORT,RFM12_SCK
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		cbi		RFM12_PORT,RFM12_SCK		

		lsl		ZL
		rol		ZH

		dec		TEMP_1
		brne	RFM12_send_cmd_loop

		sbi		RFM12_PORT,RFM12_CS
		
		ret

;***************************** RFM12 Init **********************************
; Sub: Initialisiert den RFM12
; Parameter:	-
; Return:		-
; Scratch-Reg:	Z, TEMP_0, TEMP_1
;***************************************************************************

RFM12_init:
		load_p	Z,0x0000			; Status read
		rcall	RFM12_send_cmd

		load_p	Z,0xC0E0			; CLK-Frequenz / Batterie-Level/abgeschaltet
		rcall	RFM12_send_cmd

		load_p	Z,0x80D7			; FIFO ein, C = 11,5pF
		rcall	RFM12_send_cmd

		load_p	Z,0xC2AB			; Thresold, Filter
		rcall	RFM12_send_cmd

		load_p	Z,0xCA82			; FIFO-Level,Fifo Fill an, dr aus 
		rcall	RFM12_send_cmd

		load_p	Z,0xE000			; WakeUp aus 
		rcall	RFM12_send_cmd

		load_p	Z,0xC800			; Low Duty Cycle aus, 
		rcall	RFM12_send_cmd

		load_p	Z,0xC4F3			; AFC
		rcall	RFM12_send_cmd

;		load_p	Z,0xA620			; Frequenz
		load_p	Z,RFM_FREQ_SELECT	; Frequenz:434.002350 MHz 0xA641		
		rcall	RFM12_send_cmd

		load_p	Z,0x948C			; Bandbreite
		rcall	RFM12_send_cmd

		load_p	Z,RFM_BAUD_SELECT

		rcall	RFM12_send_cmd

		load_p	Z,RFM_PW_SET
		rcall	RFM12_send_cmd

		ret

;************************** RFM12 Empfang Daten ****************************
; Sub: Liest Datenbyte vom RFM12
; Parameter:	-
; Return:		TEMP_0 Anzahl
; Scratch-Reg:	TEMP_0, TEMP_1
;***************************************************************************

RFM12_read_fifo:
		load_p	Y,DATA_BUF
		mov		TEMP_2,TEMP_0		; Anzahl merken

		cbi		RFM12_PORT,RFM12_SCK
		cbi		RFM12_PORT,RFM12_SDI

RFM12_read_fifo_loop:
		cbi		RFM12_PORT,RFM12_CS

RFM12_read_fifo_wait:
		sbis	RFM12_PIN,RFM12_SDO
		rjmp	RFM12_read_fifo_wait

		ldi		TEMP_1,8
		ldi		TEMP_0,0xB0					; Receive FIFO

RFM12_read_fifo_cmd:
		cbi		RFM12_PORT,RFM12_SDI
		sbrc	TEMP_0,7
		sbi		RFM12_PORT,RFM12_SDI

		nop
		nop
		sbi		RFM12_PORT,RFM12_SCK
		nop
		nop
		nop
		nop
		nop
		nop
		cbi		RFM12_PORT,RFM12_SCK
		lsl		TEMP_0
		nop
		dec		TEMP_1
		brne	RFM12_read_fifo_cmd

RFM12_read_fifo_daten_start:		
		clr		TEMP_0
		ldi		TEMP_1,8

RFM12_read_fifo_daten:
		lsl		TEMP_0
		sbic	RFM12_PIN,RFM12_SDO
		sbr		TEMP_0,0b00000001

		sbi		RFM12_PORT,RFM12_SCK
		nop
		nop
		nop
		nop
		cbi		RFM12_PORT,RFM12_SCK
		dec		TEMP_1
		brne	RFM12_read_fifo_daten

		st		Y+,TEMP_0
		sbi		RFM12_PORT,RFM12_CS

		dec		TEMP_2
		brne	RFM12_read_fifo_loop
	
		load_p	Z,0xCA80			; disable FIFO
		rcall	RFM12_send_cmd						
		
		ret

;************************** RFM12 Sende Daten ****************************
; Sub: Sendet Datenbyte zum RFM12
; Parameter:	-
; Return:		
; Scratch-Reg:	TEMP_0, TEMP_1, TEMP_2
;***************************************************************************

RFM12_write_fifo:
;		load_p	Z,DATA_BUF
		load_p	Z,(2*text_rfm_test)
		mov		TEMP_2,TEMP_0		; Anzahl merken

		cbi		RFM12_PORT,RFM12_SCK
		cbi		RFM12_PORT,RFM12_SDI
		cbi		RFM12_PORT,RFM12_CS

RFM12_write_fifo_loop:		
		sbis	RFM12_PIN,RFM12_SDO
		rjmp	RFM12_write_fifo_loop

		ldi		TEMP_1,8
		ldi		TEMP_0,0xB8					; Send FIFO

RFM12_write_fifo_cmd:
		cbi		RFM12_PORT,RFM12_SDI
		sbrc	TEMP_0,7
		sbi		RFM12_PORT,RFM12_SDI

		nop
		nop
		sbi		RFM12_PORT,RFM12_SCK
		nop
		nop
		nop
		nop
		nop
		nop
		cbi		RFM12_PORT,RFM12_SCK
		lsl		TEMP_0
		nop
		dec		TEMP_1
		brne	RFM12_write_fifo_cmd

RFM12_write_fifo_daten_start:		
		sbis	RFM12_PIN,RFM12_SDO
		rjmp	RFM12_write_fifo_daten_start
		
		ld		TEMP_0,Z+
		ldi		TEMP_1,8

RFM12_write_fifo_daten:
		cbi		RFM12_PORT,RFM12_SDI
		sbrc	TEMP_0,7
		sbi		RFM12_PORT,RFM12_SDI

		nop
		nop
		sbi		RFM12_PORT,RFM12_SCK
		nop
		nop
		nop
		nop
		nop
		nop
		cbi		RFM12_PORT,RFM12_SCK
		lsl		TEMP_0
		nop
		dec		TEMP_1
		brne	RFM12_write_fifo_daten
		
		
		
		dec		TEMP_2
		brne	RFM12_write_fifo_loop

		sbi		RFM12_PORT,RFM12_CS

		load_p	Z,0xCA80			; disable FIFO
		rcall	RFM12_send_cmd						
		
		load_p	Z,0x8238			; Sender aus
		rcall	RFM12_send_cmd
		ret

;************************** RFM12 Enable Radio ****************************

; Sub: Schaltet das Radio ein RFM12

; Parameter:	

; Return:		

; Scratch-Reg:	TEMP_0

;***************************************************************************


RFM12_radio_ein:

		load_p	Z,0x82C8			; Empfnger ein
		rcall	RFM12_send_cmd

		load_p	Z,0xCA80			; set FIFO mode
		rcall	RFM12_send_cmd

		load_p	Z,0xCA82			; enable FIFO,dr aus
		rcall	RFM12_send_cmd

		

		ret	; 

;************************** RFM12 Enable Transmitter ***********************

; Sub: Schaltet den Sender ein RFM12

; Parameter:	

; Return:		

; Scratch-Reg:	TEMP_0

;***************************************************************************


RFM12_sender_ein:

		load_p	Z,0x82F8			; Sender an
		rcall	RFM12_send_cmd
		
		load_p	Z,0xCA80			; set FIFO mode
		rcall	RFM12_send_cmd

		load_p	Z,0xCA82			; enable FIFO,dr aus
		rcall	RFM12_send_cmd

		

		ret	; 
