;************************** UART Routinen *************************

;******************************* Sub-Debug *********************************
; Sub: Sendet Text aus Programm-Flash ber die serielle Schnittstelle (USB)
; Parameter:	Z enthlt den Zeiger auf den Text im ROM
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_send_text:
		push	ZH
		push	ZL
		push	TEMP_0

uart_send_text_loop:
        lpm		TEMP_0,Z+
		tst		TEMP_0
		breq	uart_send_text_end					; Ende

        rcall   uart_send_byte

		rjmp    uart_send_text_loop					; weiter

uart_send_text_end:
		pop		TEMP_0
 		pop		ZL
		pop		ZH

        ret

;******************************* Sub-Debug *********************************
; Sub: Sendet NL/CR ber die serielle Schnittstelle (USB)
; Parameter:	-
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_send_newline:
        push    TEMP_0

		ldi     TEMP_0,$0A
        rcall   uart_send_byte
		ldi     TEMP_0,$0D
        rcall   uart_send_byte

        pop     TEMP_0

		ret

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 als HEX-Dump ber UART (USB)
; Parameter: 	TEMP_0
; Return:      	-
; Scratch-Reg:	-
;***************************************************************************

uart_send_byte_hex:
        push TEMP_0
		push TEMP_1

        ldi  TEMP_1,7
        push TEMP_0
		swap TEMP_0
		andi TEMP_0,$0F
		ori  TEMP_0,$30								; nach Ziffer
		cpi  TEMP_0,$3A								; 0...9
    	brlo wb_1
        add  TEMP_0,TEMP_1                 			; A...F
wb_1:
        rcall uart_send_byte
		pop  TEMP_0
		andi TEMP_0,$0F
		ori  TEMP_0,$30                   			; nach Ziffer
		cpi  TEMP_0,$3A                   			; 0...9
		brlo wb_2
        add  TEMP_0,TEMP_1                			; A...F
wb_2:
        rcall uart_send_byte

        pop   TEMP_1
		pop   TEMP_0

        ret

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 ber UART (USB)
; Parameter:	TEMP_0 Datenbyte
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_send_byte:
		sbis    UCSRA,UDRE                  		; Send-Register leer ?
        rjmp    uart_send_byte	    	    		; nein

        out     UDR,TEMP_0

        ret

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 ber UART (USB)
; Parameter:	TEMP_0 Datenbyte
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_send_byte_bin:
		push	TEMP_0
		push	TEMP_1
		push	TEMP_2

		ldi		TEMP_2,8
        mov		TEMP_1,TEMP_0

uart_send_byte_bin_loop:
		clr		TEMP_0
		lsl		TEMP_1
		rol		TEMP_0
		ori		TEMP_0,'0'
		rcall	uart_send_byte
		dec		TEMP_2
		brne	uart_send_byte_bin_loop

		pop		TEMP_2
		pop		TEMP_1
		pop		TEMP_0

        ret

;******************************* Sub-UART **********************************
; Sub: Empfngt Byte ber UART (USB)
; Parameter:    -
; Return:       TEMP_0 Datenbyte wenn FLAG_PROG gesetzt ist
; Scratch-Reg:  -
;***************************************************************************

uart_receive_byte:	
		sbis	UCSRA,RXC				        ; warten auf Parameter-Byte
		rjmp 	uart_receive_byte				; nein

        in		TEMP_0,UDR						; holen

uart_receive_byte_end:
		ret

