;******************************* Sub-Debug *********************************
; Sub: Sendet Text aus Programm-Flash ber die serielle Schnittstelle (USB)
; Parameter:	Z enthlt den Zeiger auf den Text im ROM
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_send_text:
		push	TEMP_0
		push	TEMP_1
		push	TEMP_2
uart_send_text_loop:
        lpm		TEMP_0,Z+
		tst		TEMP_0
		breq	uart_send_text_end					; Ende

        rcall   putchar

		rjmp    uart_send_text_loop					; weiter

uart_send_text_end:
		
		pop		TEMP_2
		pop		TEMP_1
		pop		TEMP_0
 		

        ret
;***************************************************************************
;*
;* "putchar"
;*
;* This subroutine transmits the byte stored in the "TEMP_0" register
;* The number of stop bits used is set with the sb constant
;*
;* Number of words	:14 including return
;* Number of cycles	:Depens on bit rate
;* Low registers used	:None
;* High registers used	:2 (bitcnt TEMP_1,Txbyte=TEMP_0)
;* Pointers used	:None
;*
;***************************************************************************


putchar:	
		ldi			TEMP_1,9+UART_ANZAHL_STOPPBIT			;1+8+UART_ANZAHL_STOPPBIT
		com			TEMP_0				;Inverte everything
		sec								;Start bit

putchar0:	
		brcc		putchar1			;If carry set
		cbi			PORTA,TxD			;send a '0'
		rjmp		putchar2			;else	

putchar1:	
		sbi			PORTA,TxD			;send a '1'
		nop

putchar2:
		rcall 		UART_delay			;One bit delay
		rcall 		UART_delay

		lsr			TEMP_0				;Get next bit
		dec			TEMP_1				;If not all bit sent
		brne		putchar0			;send next
								;else
		ret						;return


;***************************************************************************
;*
;* "getchar"
;*
;* This subroutine receives one byte and returns it in the "TEMP_0" register
;*
;* Number of words	:14 including return
;* Number of cycles	:Depens on when data arrives
;* Low registers used	:None
;* High registers used	:2 (bitcnt TEMP_1,Rxbyte TEMP_0)
;* Pointers used	:None
;*
;***************************************************************************

getchar:	
		ldi 		TEMP_1,8+UART_ANZAHL_STOPPBIT			;8 data bit + 1 stop bit
		
getchar1:	
		sbic 		PINA,RxD			;Wait for start bit
		rjmp 		getchar1

		rcall 		UART_delay			;0.5 bit delay

getchar2:	
		rcall 		UART_delay			;1 bit delay
		rcall 		UART_delay		

		clc								;clear carry
		sbic 		PINA,RxD			;if RX pin high
		sec						;

		dec 		TEMP_1				;If bit is stop bit
		breq 		getchar3			;return
							        	;else
		ror 		TEMP_0				;shift bit into Rxbyte
		rjmp 		getchar2			;go get next

getchar3:	
		ret


;***************************************************************************
;*
;* "UART_delay"
;*
;* This delay subroutine generates the required delay between the bits when
;* transmitting and receiving bytes. The total execution time is set by the
;* constant "b":
;*
;*	3b + 7 cycles (including rcall and ret)
;*
;* Number of words	:4 including return
;* Low registers used	:None
;* High registers used	:1 (TEMP_0)
;* Pointers used	:None
;*
;***************************************************************************

UART_delay:	
		ldi		TEMP_2,UART_BAUD_DELAY

UART_delay1:
		dec		TEMP_2
		brne		UART_delay1

		ret
	
 
